@layer rdg.MeasuringCell{.mlln6zg7-0-0-beta-40{contain:strict;grid-row:1;visibility:hidden}}@layer rdg.Cell{.cj343x07-0-0-beta-40{position:relative;padding-block:0;padding-inline:8px;border-inline-end:1px solid var(--rdg-border-color);border-block-end:1px solid var(--rdg-border-color);grid-row-start:var(--rdg-grid-row-start);background-color:inherit;white-space:nowrap;overflow:clip;text-overflow:ellipsis;outline:none}.cj343x07-0-0-beta-40[aria-selected=true]{outline:2px solid var(--rdg-selection-color);outline-offset:-2px}.csofj7r7-0-0-beta-40{position:sticky;z-index:1}.ch2wcw87-0-0-beta-40{box-shadow:calc(2px * var(--rdg-sign)) 0 5px -2px hsla(0,0%,53%,.3)}}@layer rdg.CheckboxLabel{.c1bn88vv7-0-0-beta-40{cursor:pointer;display:flex;align-items:center;justify-content:center;position:absolute;inset:0;margin-inline-end:1px}}@layer rdg.CheckboxInput{.c1qt073l7-0-0-beta-40{all:unset}}@layer rdg.CheckboxIcon{.cf71kmq7-0-0-beta-40{content:"";inline-size:20px;block-size:20px;border:2px solid var(--rdg-border-color);background-color:var(--rdg-background-color)}.c1qt073l7-0-0-beta-40:checked+.cf71kmq7-0-0-beta-40{background-color:var(--rdg-checkbox-color);outline:4px solid var(--rdg-background-color);outline-offset:-6px}.c1qt073l7-0-0-beta-40:focus+.cf71kmq7-0-0-beta-40{border-color:var(--rdg-checkbox-focus-color)}}@layer rdg.CheckboxLabel{.c1lwve4p7-0-0-beta-40{cursor:default}.c1lwve4p7-0-0-beta-40 .cf71kmq7-0-0-beta-40{border-color:var(--rdg-checkbox-disabled-border-color);background-color:var(--rdg-checkbox-disabled-background-color)}}@layer rdg.GroupCellContent{.g1s9ylgp7-0-0-beta-40{outline:none}}@layer rdg.GroupCellCaret{.cz54e4y7-0-0-beta-40{margin-inline-start:4px;stroke:currentColor;stroke-width:1.5px;fill:transparent;vertical-align:middle}.cz54e4y7-0-0-beta-40>path{transition:d .1s}}@layer rdg.DragHandle{.c1w9bbhr7-0-0-beta-40{--rdg-drag-handle-size:8px;z-index:0;cursor:move;inline-size:var(--rdg-drag-handle-size);block-size:var(--rdg-drag-handle-size);background-color:var(--rdg-selection-color);place-self:end}.c1w9bbhr7-0-0-beta-40:hover{--rdg-drag-handle-size:16px;border:2px solid var(--rdg-selection-color);background-color:var(--rdg-background-color)}.c1creorc7-0-0-beta-40{z-index:1;position:sticky}}@layer rdg.EditCell{.cis5rrm7-0-0-beta-40{padding:0}}@layer rdg.SortableHeaderCell{.h44jtk67-0-0-beta-40{display:flex}}@layer rdg.SortableHeaderCellName{.hcgkhxz7-0-0-beta-40{flex-grow:1;overflow:clip;text-overflow:ellipsis}}@layer rdg.HeaderCell{.c6l2wv17-0-0-beta-40{cursor:pointer}.c1kqdw7y7-0-0-beta-40{touch-action:none}.r1y6ywlx7-0-0-beta-40{cursor:col-resize;position:absolute;inset-block-start:0;inset-inline-end:0;inset-block-end:0;inline-size:10px}}.c1bezg5o7-0-0-beta-40{opacity:.5}.c1vc96037-0-0-beta-40{background-color:var(--rdg-header-draggable-background-color)}@layer rdg.Row{.r1upfr807-0-0-beta-40{display:contents;line-height:var(--rdg-row-height);background-color:var(--rdg-background-color)}.r1upfr807-0-0-beta-40:hover{background-color:var(--rdg-row-hover-background-color)}.r1upfr807-0-0-beta-40[aria-selected=true]{background-color:var(--rdg-row-selected-background-color)}.r1upfr807-0-0-beta-40[aria-selected=true]:hover{background-color:var(--rdg-row-selected-hover-background-color)}}@layer rdg.FocusSink{.r190mhd37-0-0-beta-40{outline:2px solid var(--rdg-selection-color);outline-offset:-2px}.r139qu9m7-0-0-beta-40:before{content:"";display:inline-block;height:100%;position:sticky;inset-inline-start:0;border-inline-start:2px solid var(--rdg-selection-color)}}@layer rdg.HeaderRow{.h10tskcx7-0-0-beta-40{display:contents;line-height:var(--rdg-header-row-height);background-color:var(--rdg-header-background-color);font-weight:700}.h10tskcx7-0-0-beta-40>.cj343x07-0-0-beta-40{z-index:2;position:sticky}.h10tskcx7-0-0-beta-40>.csofj7r7-0-0-beta-40{z-index:3}}@layer rdg.Cell{.c6ra8a37-0-0-beta-40,.cq910m07-0-0-beta-40{background-color:#ccf}.cq910m07-0-0-beta-40.c6ra8a37-0-0-beta-40{background-color:#99f}}@layer rdg.SortIcon{.a3ejtar7-0-0-beta-40{fill:currentColor}.a3ejtar7-0-0-beta-40>path{transition:d .1s}}@layer rdg{@layer Defaults,
      FocusSink,
      CheckboxInput,
      CheckboxIcon,
      CheckboxLabel,
      Cell,
      HeaderCell,
      SummaryCell,
      EditCell,
      Row,
      HeaderRow,
      SummaryRow,
      GroupedRow,
      Root;@layer Defaults{.rnvodz57-0-0-beta-40 *,.rnvodz57-0-0-beta-40 :after,.rnvodz57-0-0-beta-40 :before{box-sizing:inherit}}@layer Root{.rnvodz57-0-0-beta-40{--rdg-color:#000;--rdg-border-color:#ddd;--rdg-summary-border-color:#aaa;--rdg-background-color:#fff;--rdg-header-background-color:#f9f9f9;--rdg-header-draggable-background-color:#e7e7e7;--rdg-row-hover-background-color:#f5f5f5;--rdg-row-selected-background-color:#dbecfa;--rdg-row-selected-hover-background-color:#c9e3f8;--rdg-checkbox-color:#005194;--rdg-checkbox-focus-color:#61b8ff;--rdg-checkbox-disabled-border-color:#ccc;--rdg-checkbox-disabled-background-color:#ddd;--rdg-selection-color:#66afe9;--rdg-font-size:14px;display:grid;color-scheme:var(--rdg-color-scheme,light dark);contain:content;content-visibility:auto;block-size:350px;border:1px solid var(--rdg-border-color);box-sizing:border-box;overflow:auto;background-color:var(--rdg-background-color);color:var(--rdg-color);font-size:var(--rdg-font-size)}.rnvodz57-0-0-beta-40:before{content:"";grid-column:1/-1;grid-row:1/-1}.rnvodz57-0-0-beta-40.rdg-dark{--rdg-color-scheme:dark;--rdg-color:#ddd;--rdg-border-color:#444;--rdg-summary-border-color:#555;--rdg-background-color:#212121;--rdg-header-background-color:#1b1b1b;--rdg-header-draggable-background-color:#2d2d2d;--rdg-row-hover-background-color:#171717;--rdg-row-selected-background-color:#1a73bc;--rdg-row-selected-hover-background-color:#1768ab;--rdg-checkbox-color:#94cfff;--rdg-checkbox-focus-color:#c7e6ff;--rdg-checkbox-disabled-border-color:#000;--rdg-checkbox-disabled-background-color:#333}.rnvodz57-0-0-beta-40.rdg-light{--rdg-color-scheme:light}@media (prefers-color-scheme:dark){.rnvodz57-0-0-beta-40:not(.rdg-light){--rdg-color:#ddd;--rdg-border-color:#444;--rdg-summary-border-color:#555;--rdg-background-color:#212121;--rdg-header-background-color:#1b1b1b;--rdg-header-draggable-background-color:#2d2d2d;--rdg-row-hover-background-color:#171717;--rdg-row-selected-background-color:#1a73bc;--rdg-row-selected-hover-background-color:#1768ab;--rdg-checkbox-color:#94cfff;--rdg-checkbox-focus-color:#c7e6ff;--rdg-checkbox-disabled-border-color:#000;--rdg-checkbox-disabled-background-color:#333}}}}@layer rdg.Root{.vlqv91k7-0-0-beta-40{-webkit-user-select:none;-moz-user-select:none;user-select:none}.vlqv91k7-0-0-beta-40 .r1upfr807-0-0-beta-40{cursor:move}}@layer rdg.FocusSink{.f1lsfrzw7-0-0-beta-40{grid-column:1/-1;pointer-events:none;z-index:1}.f1cte0lg7-0-0-beta-40{z-index:3}}@layer rdg.SummaryCell{.s8wc6fl7-0-0-beta-40{inset-block-start:var(--rdg-summary-row-top);inset-block-end:var(--rdg-summary-row-bottom)}}@layer rdg.SummaryRow{.skuhp557-0-0-beta-40{line-height:var(--rdg-summary-row-height)}.skuhp557-0-0-beta-40>.cj343x07-0-0-beta-40{position:sticky}.tf8l5ub7-0-0-beta-40>.cj343x07-0-0-beta-40{z-index:2}.tf8l5ub7-0-0-beta-40>.csofj7r7-0-0-beta-40{z-index:3}.tb9ughf7-0-0-beta-40>.cj343x07-0-0-beta-40{border-block-end:2px solid var(--rdg-summary-border-color)}.b1yssfnt7-0-0-beta-40>.cj343x07-0-0-beta-40{border-block-start:2px solid var(--rdg-summary-border-color)}}@layer rdg.GroupedRow{.g1yxluv37-0-0-beta-40:not([aria-selected=true]){background-color:var(--rdg-header-background-color)}.g1yxluv37-0-0-beta-40>.cj343x07-0-0-beta-40:not(:last-child):not(.ch2wcw87-0-0-beta-40){border-inline-end:none}}@layer rdg.TextEditor{.t7vyx3i7-0-0-beta-40{-webkit-appearance:none;-moz-appearance:none;appearance:none;box-sizing:border-box;inline-size:100%;block-size:100%;padding-block:0;padding-inline:6px;border:2px solid #ccc;vertical-align:top;color:var(--rdg-color);background-color:var(--rdg-background-color);font-family:inherit;font-size:var(--rdg-font-size)}.t7vyx3i7-0-0-beta-40:focus{border-color:var(--rdg-selection-color);outline:none}.t7vyx3i7-0-0-beta-40::-moz-placeholder{color:#999;opacity:1}.t7vyx3i7-0-0-beta-40::placeholder{color:#999;opacity:1}}.index-column{color:grey;text-align:right}.diff-header-added{background-color:#3aa53a;color:#fff}.diff-cell-added{background-color:#cefece;color:#000}.diff-header-removed{background-color:#f66;color:#fff}.diff-cell-modified,.diff-cell-removed{background-color:#ffc5c5;color:#000}.cell-show-context-menu{overflow:visible}.grid-header:hover .key-icon,.grid-header:hover .pin-icon{display:block}.rdg-cell{border-width:0 1px 1px 0;border-style:solid;border-color:var(--rdg-border-color)}.rdg-row:hover .row-context-menu{visibility:visible;width:auto}.column-index-added{text-align:right;background-color:#ccffd8}.column-body-added{background-color:#e6ffec}.column-index-removed{text-align:right;background-color:#ffd7d5}.column-body-removed{background-color:#ffebe9}.column-index-reordered{text-align:right;background-color:#ffad15}.column-body-reordered{background-color:#fff}.column-index-normal{text-align:right;background-color:#f9f9f9}.column-body-normal{background-color:#fff}.column-body-type-changed{background-color:#ffd7d5}.gutter{background-color:#eee;background-repeat:no-repeat;background-position:50%}.gutter:hover{background-color:orange}.gutter.gutter-vertical{background-image:url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAB4AAAAFAQMAAABo7865AAAABlBMVEVHcEzMzMzyAv2sAAAAAXRSTlMAQObYZgAAABBJREFUeF5jOAMEEAIEEFwAn3kMwcB6I2AAAAAASUVORK5CYII=");cursor:row-resize}.gutter.gutter-horizontal{background-image:url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAeCAYAAADkftS9AAAAIklEQVQoU2M4c+bMfxAGAgYYmwGrIIiDjrELjpo5aiZeMwF+yNnOs5KSvgAAAABJRU5ErkJggg==");cursor:col-resize}.react-flow{direction:ltr}.react-flow__container{position:absolute;width:100%;height:100%;top:0;left:0}.react-flow__pane{z-index:1;cursor:grab}.react-flow__pane.selection{cursor:pointer}.react-flow__pane.dragging{cursor:grabbing}.react-flow__viewport{transform-origin:0 0;z-index:2;pointer-events:none}.react-flow__renderer{z-index:4}.react-flow__selection{z-index:6}.react-flow__nodesselection-rect:focus,.react-flow__nodesselection-rect:focus-visible{outline:none}.react-flow .react-flow__edges{pointer-events:none;overflow:visible}.react-flow__connection-path,.react-flow__edge-path{stroke:#b1b1b7;stroke-width:1;fill:none}.react-flow__edge{pointer-events:visibleStroke;cursor:pointer}.react-flow__edge.animated path{stroke-dasharray:5;animation:dashdraw .5s linear infinite}.react-flow__edge.animated path.react-flow__edge-interaction{stroke-dasharray:none;animation:none}.react-flow__edge.inactive{pointer-events:none}.react-flow__edge.selected,.react-flow__edge:focus,.react-flow__edge:focus-visible{outline:none}.react-flow__edge.selected .react-flow__edge-path,.react-flow__edge:focus .react-flow__edge-path,.react-flow__edge:focus-visible .react-flow__edge-path{stroke:#555}.react-flow__edge-textwrapper{pointer-events:all}.react-flow__edge-textbg{fill:#fff}.react-flow__edge .react-flow__edge-text{pointer-events:none;-webkit-user-select:none;-moz-user-select:none;user-select:none}.react-flow__connection{pointer-events:none}.react-flow__connection .animated{stroke-dasharray:5;animation:dashdraw .5s linear infinite}.react-flow__connectionline{z-index:1001}.react-flow__nodes{pointer-events:none;transform-origin:0 0}.react-flow__node{position:absolute;-webkit-user-select:none;-moz-user-select:none;user-select:none;pointer-events:all;transform-origin:0 0;box-sizing:border-box;cursor:grab}.react-flow__node.dragging{cursor:grabbing}.react-flow__nodesselection{z-index:3;transform-origin:left top;pointer-events:none}.react-flow__nodesselection-rect{position:absolute;pointer-events:all;cursor:grab}.react-flow__handle{position:absolute;pointer-events:none;min-width:5px;min-height:5px;width:6px;height:6px;background:#1a192b;border:1px solid #fff;border-radius:100%}.react-flow__handle.connectionindicator{pointer-events:all;cursor:crosshair}.react-flow__handle-bottom{top:auto;left:50%;bottom:-4px;transform:translate(-50%)}.react-flow__handle-top{left:50%;top:-4px;transform:translate(-50%)}.react-flow__handle-left{top:50%;left:-4px;transform:translateY(-50%)}.react-flow__handle-right{right:-4px;top:50%;transform:translateY(-50%)}.react-flow__edgeupdater{cursor:move;pointer-events:all}.react-flow__panel{position:absolute;z-index:5;margin:15px}.react-flow__panel.top{top:0}.react-flow__panel.bottom{bottom:0}.react-flow__panel.left{left:0}.react-flow__panel.right{right:0}.react-flow__panel.center{left:50%;transform:translateX(-50%)}.react-flow__attribution{font-size:10px;background:hsla(0,0%,100%,.5);padding:2px 3px;margin:0}.react-flow__attribution a{text-decoration:none;color:#999}@keyframes dashdraw{0%{stroke-dashoffset:10}}.react-flow__edgelabel-renderer{position:absolute;width:100%;height:100%;pointer-events:none;-webkit-user-select:none;-moz-user-select:none;user-select:none}.react-flow__edge.updating .react-flow__edge-path{stroke:#777}.react-flow__edge-text{font-size:10px}.react-flow__node.selectable:focus,.react-flow__node.selectable:focus-visible{outline:none}.react-flow__node-default,.react-flow__node-group,.react-flow__node-input,.react-flow__node-output{padding:10px;border-radius:3px;width:150px;font-size:12px;color:#222;text-align:center;border:1px solid #1a192b;background-color:#fff}.react-flow__node-default.selectable:hover,.react-flow__node-group.selectable:hover,.react-flow__node-input.selectable:hover,.react-flow__node-output.selectable:hover{box-shadow:0 1px 4px 1px rgba(0,0,0,.08)}.react-flow__node-default.selectable.selected,.react-flow__node-default.selectable:focus,.react-flow__node-default.selectable:focus-visible,.react-flow__node-group.selectable.selected,.react-flow__node-group.selectable:focus,.react-flow__node-group.selectable:focus-visible,.react-flow__node-input.selectable.selected,.react-flow__node-input.selectable:focus,.react-flow__node-input.selectable:focus-visible,.react-flow__node-output.selectable.selected,.react-flow__node-output.selectable:focus,.react-flow__node-output.selectable:focus-visible{box-shadow:0 0 0 .5px #1a192b}.react-flow__node-group{background-color:hsla(0,0%,94%,.25)}.react-flow__nodesselection-rect,.react-flow__selection{background:rgba(0,89,220,.08);border:1px dotted rgba(0,89,220,.8)}.react-flow__nodesselection-rect:focus,.react-flow__nodesselection-rect:focus-visible,.react-flow__selection:focus,.react-flow__selection:focus-visible{outline:none}.react-flow__controls{box-shadow:0 0 2px 1px rgba(0,0,0,.08)}.react-flow__controls-button{background:#fefefe;border:none;border-bottom:1px solid #eee;box-sizing:content-box;display:flex;justify-content:center;align-items:center;width:16px;height:16px;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;user-select:none;padding:5px}.react-flow__controls-button:hover{background:#f4f4f4}.react-flow__controls-button svg{width:100%;max-width:12px;max-height:12px}.react-flow__controls-button:disabled{pointer-events:none}.react-flow__controls-button:disabled svg{fill-opacity:.4}.react-flow__minimap{background-color:#fff}.react-flow__minimap svg{display:block}.react-flow__resize-control{position:absolute}.react-flow__resize-control.left,.react-flow__resize-control.right{cursor:ew-resize}.react-flow__resize-control.bottom,.react-flow__resize-control.top{cursor:ns-resize}.react-flow__resize-control.bottom.right,.react-flow__resize-control.top.left{cursor:nwse-resize}.react-flow__resize-control.bottom.left,.react-flow__resize-control.top.right{cursor:nesw-resize}.react-flow__resize-control.handle{width:4px;height:4px;border:1px solid #fff;border-radius:1px;background-color:#3367d9;transform:translate(-50%,-50%)}.react-flow__resize-control.handle.left{left:0;top:50%}.react-flow__resize-control.handle.right{left:100%;top:50%}.react-flow__resize-control.handle.top{left:50%;top:0}.react-flow__resize-control.handle.bottom{left:50%;top:100%}.react-flow__resize-control.handle.bottom.left,.react-flow__resize-control.handle.top.left{left:0}.react-flow__resize-control.handle.bottom.right,.react-flow__resize-control.handle.top.right{left:100%}.react-flow__resize-control.line{border:0 solid #3367d9}.react-flow__resize-control.line.left,.react-flow__resize-control.line.right{width:1px;transform:translate(-50%);top:0;height:100%}.react-flow__resize-control.line.left{left:0;border-left-width:1px}.react-flow__resize-control.line.right{left:100%;border-right-width:1px}.react-flow__resize-control.line.bottom,.react-flow__resize-control.line.top{height:1px;transform:translateY(-50%);left:0;width:100%}.react-flow__resize-control.line.top{top:0;border-top-width:1px}.react-flow__resize-control.line.bottom{border-bottom-width:1px;top:100%}.node-unhighlight{filter:opacity(.2) grayscale(50%)}.node-highlight{border-width:1;border-color:"orange";box-shadow:"0px 5px 15px #00000040"}@font-face{font-family:Montserrat;font-style:normal;font-display:swap;font-weight:800;src:url(/_next/static/media/montserrat-cyrillic-ext-800-normal.83d3554a.woff2) format("woff2"),url(/_next/static/media/montserrat-cyrillic-ext-800-normal.8a4a9645.woff) format("woff");unicode-range:u+0460-052f,u+1c80-1c88,u+20b4,u+2de0-2dff,u+a640-a69f,u+fe2e-fe2f}@font-face{font-family:Montserrat;font-style:normal;font-display:swap;font-weight:800;src:url(/_next/static/media/montserrat-cyrillic-800-normal.c03acce9.woff2) format("woff2"),url(/_next/static/media/montserrat-cyrillic-800-normal.5c8b3804.woff) format("woff");unicode-range:u+0301,u+0400-045f,u+0490-0491,u+04b0-04b1,u+2116}@font-face{font-family:Montserrat;font-style:normal;font-display:swap;font-weight:800;src:url(/_next/static/media/montserrat-vietnamese-800-normal.dfbbc7ba.woff2) format("woff2"),url(/_next/static/media/montserrat-vietnamese-800-normal.0bd05041.woff) format("woff");unicode-range:u+0102-0103,u+0110-0111,u+0128-0129,u+0168-0169,u+01a0-01a1,u+01af-01b0,u+0300-0301,u+0303-0304,u+0308-0309,u+0323,u+0329,u+1ea0-1ef9,u+20ab}@font-face{font-family:Montserrat;font-style:normal;font-display:swap;font-weight:800;src:url(/_next/static/media/montserrat-latin-ext-800-normal.85728e97.woff2) format("woff2"),url(/_next/static/media/montserrat-latin-ext-800-normal.40cba3c6.woff) format("woff");unicode-range:u+0100-02af,u+0304,u+0308,u+0329,u+1e00-1e9f,u+1ef2-1eff,u+2020,u+20a0-20ab,u+20ad-20c0,u+2113,u+2c60-2c7f,u+a720-a7ff}@font-face{font-family:Montserrat;font-style:normal;font-display:swap;font-weight:800;src:url(/_next/static/media/montserrat-latin-800-normal.388abfc2.woff2) format("woff2"),url(/_next/static/media/montserrat-latin-800-normal.e7b55ea8.woff) format("woff");unicode-range:u+00??,u+0131,u+0152-0153,u+02bb-02bc,u+02c6,u+02da,u+02dc,u+0304,u+0308,u+0329,u+2000-206f,u+2074,u+20ac,u+2122,u+2191,u+2193,u+2212,u+2215,u+feff,u+fffd}[data-simplebar]{position:relative;flex-direction:column;flex-wrap:wrap;justify-content:flex-start;align-content:flex-start;align-items:flex-start}.simplebar-wrapper{overflow:hidden;width:inherit;height:inherit;max-width:inherit;max-height:inherit}.simplebar-mask{direction:inherit;overflow:hidden;width:auto!important;height:auto!important;z-index:0}.simplebar-mask,.simplebar-offset{position:absolute;padding:0;margin:0;left:0;top:0;bottom:0;right:0}.simplebar-offset{direction:inherit!important;box-sizing:inherit!important;resize:none!important;-webkit-overflow-scrolling:touch}.simplebar-content-wrapper{direction:inherit;box-sizing:border-box!important;position:relative;display:block;height:100%;width:auto;max-width:100%;max-height:100%;overflow:auto;scrollbar-width:none;-ms-overflow-style:none}.simplebar-content-wrapper::-webkit-scrollbar,.simplebar-hide-scrollbar::-webkit-scrollbar{display:none;width:0;height:0}.simplebar-content:after,.simplebar-content:before{content:" ";display:table}.simplebar-placeholder{max-height:100%;max-width:100%;width:100%;pointer-events:none}.simplebar-height-auto-observer-wrapper{box-sizing:inherit!important;height:100%;width:100%;max-width:1px;position:relative;float:left;max-height:1px;overflow:hidden;z-index:-1;padding:0;margin:0;pointer-events:none;flex-grow:inherit;flex-shrink:0;flex-basis:0}.simplebar-height-auto-observer{box-sizing:inherit;display:block;opacity:0;top:0;left:0;height:1000%;width:1000%;min-height:1px;min-width:1px;z-index:-1}.simplebar-height-auto-observer,.simplebar-track{position:absolute;overflow:hidden;pointer-events:none}.simplebar-track{z-index:1;right:0;bottom:0}[data-simplebar].simplebar-dragging,[data-simplebar].simplebar-dragging .simplebar-content{pointer-events:none;-webkit-touch-callout:none;-webkit-user-select:none;-moz-user-select:none;user-select:none}[data-simplebar].simplebar-dragging .simplebar-track{pointer-events:all}.simplebar-scrollbar{position:absolute;left:0;right:0;min-height:10px}.simplebar-scrollbar:before{position:absolute;content:"";background:#000;border-radius:7px;opacity:0;transition:opacity .2s linear .5s}.simplebar-scrollbar.simplebar-visible:before{opacity:.5;transition-delay:0s;transition-duration:0s}.simplebar-track.simplebar-vertical{top:0;width:11px}.simplebar-scrollbar:before{top:2px;bottom:2px;left:2px;right:2px}.simplebar-track.simplebar-horizontal{left:0;height:11px}.simplebar-track.simplebar-horizontal .simplebar-scrollbar{right:auto;left:0;top:0;bottom:0;min-height:0;min-width:10px;width:auto}[data-simplebar-direction=rtl] .simplebar-track.simplebar-vertical{right:auto;left:0}.simplebar-dummy-scrollbar-size{direction:rtl;position:fixed;opacity:0;visibility:hidden;height:500px;width:500px;overflow-y:hidden;overflow-x:scroll;-ms-overflow-style:scrollbar!important}.simplebar-dummy-scrollbar-size>div{width:200%;height:200%;margin:10px 0}.simplebar-hide-scrollbar{position:fixed;left:0;visibility:hidden;overflow-y:scroll;scrollbar-width:none;-ms-overflow-style:none}