"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[147],{54517:function(t,r,e){e.d(r,{Pkc:function(){return i}});var n=e(26841);function i(t){return(0,n.w_)({tag:"svg",attr:{viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"2",strokeLinecap:"round",strokeLinejoin:"round"},child:[{tag:"path",attr:{d:"M3 3v18h18"}},{tag:"rect",attr:{width:"12",height:"4",x:"7",y:"5",rx:"1"}},{tag:"rect",attr:{width:"7",height:"4",x:"7",y:"13",rx:"1"}}]})(t)}}}]);